@@ -1,7 +1,6 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2017, Anaconda, Inc. All rights reserved.
-#
-# Powered by the Bokeh Development Team.
+# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
 #-----------------------------------------------------------------------------
@@ -25,10 +24,18 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 import logging
 log = logging.getLogger(__name__)
 
-from bokeh.util.api import general, dev ; general, dev
+#-----------------------------------------------------------------------------
+# Imports
+#-----------------------------------------------------------------------------
+
+# Standard library imports
+
+# External imports
+
+# Bokeh imports
 
 #-----------------------------------------------------------------------------
-# General API
+# Globals and constants
 #-----------------------------------------------------------------------------
 
 __all__ = (
@@ -37,8 +44,9 @@ __all__ = (
     'sampledata',
 )
 
-# configure Bokeh version
-from .util.version import __version__; __version__
+#-----------------------------------------------------------------------------
+# General API
+#-----------------------------------------------------------------------------
 
 def license():
     ''' Print the Bokeh license to the console.
@@ -51,21 +59,43 @@ def license():
     with open(join(__path__[0], 'LICENSE.txt')) as lic:
         print(lic.read())
 
-# expose sample data module
-from . import sampledata; sampledata
+#-----------------------------------------------------------------------------
+# Dev API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Private API
+#-----------------------------------------------------------------------------
 
 #-----------------------------------------------------------------------------
 # Code
 #-----------------------------------------------------------------------------
 
+# configure Bokeh version
+from .util.version import __version__; __version__
+
+# expose sample data module
+from . import sampledata; sampledata
+
 # configure Bokeh logger
 from .util import logconfig
 del logconfig
 
-# Configure warnings to always show, despite Python's active efforts to hide them from users.
+# Configure warnings to always show nice mssages, despite Python's active
+# efforts to hide them from users.
 import warnings
 from .util.warnings import BokehDeprecationWarning, BokehUserWarning
 warnings.simplefilter('always', BokehDeprecationWarning)
 warnings.simplefilter('always', BokehUserWarning)
+
+original_formatwarning = warnings.formatwarning
+def _formatwarning(message, category, filename, lineno, line=None):
+    from .util.warnings import BokehDeprecationWarning, BokehUserWarning
+    if category not in (BokehDeprecationWarning, BokehUserWarning):
+        return original_formatwarning(message, category, filename, lineno, line)
+    return "%s: %s\n" % (category.__name__, message)
+warnings.formatwarning = _formatwarning
+
+del _formatwarning
 del BokehDeprecationWarning, BokehUserWarning
 del warnings
