@@ -18,7 +18,11 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
-from collections import Iterable, OrderedDict, Sequence
+from collections import OrderedDict
+from bokeh.util.future import collections_abc # goes away with py2
+Iterable = collections_abc.Iterable # NOQA
+Sequence = collections_abc.Sequence # NOQA
+
 import difflib
 import itertools
 import re
@@ -92,7 +96,40 @@ def get_default_color(plot=None):
 # Private API
 #-----------------------------------------------------------------------------
 
-def _stack(stackers, spec0, spec1, **kw):
+def _single_stack(stackers, spec, **kw):
+    if spec in kw:
+        raise ValueError("Stack property '%s' cannot appear in keyword args" % spec)
+
+    lengths = { len(x) for x in kw.values() if isinstance(x, (list, tuple)) }
+
+    # lengths will be empty if there are no kwargs supplied at all
+    if len(lengths) > 0:
+        if len(lengths) != 1:
+            raise ValueError("Keyword argument sequences for broadcasting must all be the same lengths. Got lengths: %r" % sorted(list(lengths)))
+        if lengths.pop() != len(stackers):
+            raise ValueError("Keyword argument sequences for broadcasting must be the same length as stackers")
+
+    s = []
+
+    _kw = []
+
+    for i, val in enumerate(stackers):
+        d  = {'name': val}
+        s.append(val)
+
+        d[spec] = stack(*s)
+
+        for k, v in kw.items():
+            if isinstance(v, (list, tuple)):
+                d[k] = v[i]
+            else:
+                d[k] = v
+
+        _kw.append(d)
+
+    return _kw
+
+def _double_stack(stackers, spec0, spec1, **kw):
     for name in (spec0, spec1):
         if name in kw:
             raise ValueError("Stack property '%s' cannot appear in keyword args" % name)
@@ -526,10 +563,9 @@ def _tool_from_string(name):
 
 def _process_axis_and_grid(plot, axis_type, axis_location, minor_ticks, axis_label, rng, dim):
     axiscls, axiskw = _get_axis_class(axis_type, rng, dim)
-    if axiscls:
 
-        # this is so we can get a ticker off the axis, even if we discard it
-        axis = axiscls(plot=plot if axis_location else None, **axiskw)
+    if axiscls:
+        axis = axiscls(**axiskw)
 
         if isinstance(axis.ticker, ContinuousTicker):
             axis.ticker.num_minor_ticks = _get_num_minor_ticks(axiscls, minor_ticks)
@@ -538,7 +574,8 @@ def _process_axis_and_grid(plot, axis_type, axis_location, minor_ticks, axis_lab
         if axis_label:
             axis.axis_label = axis_label
 
-        grid = Grid(plot=plot, dimension=dim, ticker=axis.ticker); grid
+        grid = Grid(dimension=dim, ticker=axis.ticker)
+        plot.add_layout(grid, "center")
 
         if axis_location is not None:
             getattr(plot, axis_location).append(axis)
