@@ -31,7 +31,6 @@ from bokeh.models.scales import LinearScale, LogScale, CategoricalScale
 from bokeh.plotting import Figure
 
 # Module under test
-from bokeh.plotting.helpers import _get_scale,_get_range, _stack, _graph, _glyph_function, _RENDERER_ARGS, _get_axis_class
 import bokeh.plotting.helpers as bph
 
 #-----------------------------------------------------------------------------
@@ -90,41 +89,112 @@ class Test__get_legend_item_label(object):
         label = bph._get_legend_item_label(kwargs)
         assert label == {'value': 'not_a_column_label'}
 
-def test__stack_raises_when_spec_in_kwargs():
+def test__single_stack_raises_when_spec_in_kwargs():
     with pytest.raises(ValueError) as e:
-        _stack(['a', 'b'], 'foo', 'bar', foo=10)
+        bph._single_stack(['a', 'b'], 'foo', foo=10)
+
+    assert str(e).endswith("Stack property 'foo' cannot appear in keyword args")
+
+def test__single_stack_raises_when_kwargs_list_lengths_differ():
+    with pytest.raises(ValueError) as e:
+        bph._single_stack(['a', 'b'], 'foo', baz=[1, 2], quux=[3,4,5])
+
+    assert str(e).endswith("Keyword argument sequences for broadcasting must all be the same lengths. Got lengths: [2, 3]")
+
+def test__single_stack_raises_when_kwargs_list_lengths_and_stackers_lengths_differ():
+    with pytest.raises(ValueError) as e:
+        bph._single_stack(['a', 'b', 'c'], 'foo',  baz=[1, 2], quux=[3,4])
+
+    assert str(e).endswith("Keyword argument sequences for broadcasting must be the same length as stackers")
+
+def test__single_stack_broadcast_with_no_kwargs():
+    stackers = ['a', 'b', 'c', 'd']
+    kws = bph._single_stack(stackers, 'start')
+    assert len(kws) == len(stackers)
+    for i, kw in enumerate(kws):
+        assert set(['start', 'name']) == set(kw.keys())
+        assert list(kw['start']['expr'].fields) == stackers[:i+1]
+
+def test__single_stack_broadcast_with_scalar_kwargs():
+    stackers = ['a', 'b', 'c', 'd']
+    kws = bph._single_stack(stackers, 'start', foo=10, bar="baz")
+    assert len(kws) == len(stackers)
+    for i, kw in enumerate(kws):
+        assert set(['start', 'foo', 'bar', 'name']) == set(kw.keys())
+        assert list(kw['start']['expr'].fields) == stackers[:i+1]
+        assert kw['foo'] == 10
+        assert kw['bar'] == "baz"
+        assert kw['name'] == stackers[i]
+
+def test__single_stack_broadcast_with_list_kwargs():
+    stackers = ['a', 'b', 'c', 'd']
+    kws = bph._single_stack(stackers, 'start', foo=[10, 20, 30, 40], bar="baz")
+    assert len(kws) == len(stackers)
+    for i, kw in enumerate(kws):
+        assert set(['start', 'foo', 'bar', 'name']) == set(kw.keys())
+        assert list(kw['start']['expr'].fields) == stackers[:i+1]
+        assert kw['foo'] == [10, 20, 30, 40][i]
+        assert kw['bar'] == "baz"
+        assert kw['name'] == stackers[i]
+
+def test__single_stack_broadcast_name_scalar_overrides():
+    stackers = ['a', 'b', 'c', 'd']
+    kws = bph._single_stack(stackers, 'start', foo=[10, 20, 30, 40], bar="baz", name="name")
+    assert len(kws) == len(stackers)
+    for i, kw in enumerate(kws):
+        assert set(['start', 'foo', 'bar', 'name']) == set(kw.keys())
+        assert list(kw['start']['expr'].fields) == stackers[:i+1]
+        assert kw['foo'] == [10, 20, 30, 40][i]
+        assert kw['bar'] == "baz"
+        assert kw['name'] == "name"
+
+def test__single_stack_broadcast_name_list_overrides():
+    names = ["aa", "bb", "cc", "dd"]
+    stackers = ['a', 'b', 'c', 'd']
+    kws = bph._single_stack(stackers, 'start', foo=[10, 20, 30, 40], bar="baz", name=names)
+    assert len(kws) == len(stackers)
+    for i, kw in enumerate(kws):
+        assert set(['start', 'foo', 'bar', 'name']) == set(kw.keys())
+        assert list(kw['start']['expr'].fields) == stackers[:i+1]
+        assert kw['foo'] == [10, 20, 30, 40][i]
+        assert kw['bar'] == "baz"
+        assert kw['name'] == names[i]
+
+def test__double_stack_raises_when_spec_in_kwargs():
+    with pytest.raises(ValueError) as e:
+        bph._double_stack(['a', 'b'], 'foo', 'bar', foo=10)
 
     assert str(e).endswith("Stack property 'foo' cannot appear in keyword args")
 
     with pytest.raises(ValueError) as e:
-        _stack(['a', 'b'], 'foo', 'bar', bar=10)
+        bph._double_stack(['a', 'b'], 'foo', 'bar', bar=10)
 
     assert str(e).endswith("Stack property 'bar' cannot appear in keyword args")
 
-def test__stack_raises_when_kwargs_list_lengths_differ():
+def test__double_stack_raises_when_kwargs_list_lengths_differ():
     with pytest.raises(ValueError) as e:
-        _stack(['a', 'b'], 'foo', 'bar', baz=[1, 2], quux=[3,4,5])
+        bph._double_stack(['a', 'b'], 'foo', 'bar', baz=[1, 2], quux=[3,4,5])
 
     assert str(e).endswith("Keyword argument sequences for broadcasting must all be the same lengths. Got lengths: [2, 3]")
 
-def test__stack_raises_when_kwargs_list_lengths_and_stackers_lengths_differ():
+def test__double_stack_raises_when_kwargs_list_lengths_and_stackers_lengths_differ():
     with pytest.raises(ValueError) as e:
-        _stack(['a', 'b', 'c'], 'foo', 'bar', baz=[1, 2], quux=[3,4])
+        bph._double_stack(['a', 'b', 'c'], 'foo', 'bar', baz=[1, 2], quux=[3,4])
 
     assert str(e).endswith("Keyword argument sequences for broadcasting must be the same length as stackers")
 
-def test__stack_broadcast_with_no_kwargs():
+def test__double_stack_broadcast_with_no_kwargs():
     stackers = ['a', 'b', 'c', 'd']
-    kws = _stack(stackers, 'start', 'end')
+    kws = bph._double_stack(stackers, 'start', 'end')
     assert len(kws) == len(stackers)
     for i, kw in enumerate(kws):
         assert set(['start', 'end', 'name']) == set(kw.keys())
         assert list(kw['start']['expr'].fields) == stackers[:i]
         assert list(kw['end']['expr'].fields) == stackers[:(i+1)]
 
-def test__stack_broadcast_with_scalar_kwargs():
+def test__double_stack_broadcast_with_scalar_kwargs():
     stackers = ['a', 'b', 'c', 'd']
-    kws = _stack(stackers, 'start', 'end', foo=10, bar="baz")
+    kws = bph._double_stack(stackers, 'start', 'end', foo=10, bar="baz")
     assert len(kws) == len(stackers)
     for i, kw in enumerate(kws):
         assert set(['start', 'end', 'foo', 'bar', 'name']) == set(kw.keys())
@@ -134,9 +204,9 @@ def test__stack_broadcast_with_scalar_kwargs():
         assert kw['bar'] == "baz"
         assert kw['name'] == stackers[i]
 
-def test__stack_broadcast_with_list_kwargs():
+def test__double_stack_broadcast_with_list_kwargs():
     stackers = ['a', 'b', 'c', 'd']
-    kws = _stack(stackers, 'start', 'end', foo=[10, 20, 30, 40], bar="baz")
+    kws = bph._double_stack(stackers, 'start', 'end', foo=[10, 20, 30, 40], bar="baz")
     assert len(kws) == len(stackers)
     for i, kw in enumerate(kws):
         assert set(['start', 'end', 'foo', 'bar', 'name']) == set(kw.keys())
@@ -146,9 +216,9 @@ def test__stack_broadcast_with_list_kwargs():
         assert kw['bar'] == "baz"
         assert kw['name'] == stackers[i]
 
-def test__stack_broadcast_name_scalar_overrides():
+def test__double_stack_broadcast_name_scalar_overrides():
     stackers = ['a', 'b', 'c', 'd']
-    kws = _stack(stackers, 'start', 'end', foo=[10, 20, 30, 40], bar="baz", name="name")
+    kws = bph._double_stack(stackers, 'start', 'end', foo=[10, 20, 30, 40], bar="baz", name="name")
     assert len(kws) == len(stackers)
     for i, kw in enumerate(kws):
         assert set(['start', 'end', 'foo', 'bar', 'name']) == set(kw.keys())
@@ -158,10 +228,10 @@ def test__stack_broadcast_name_scalar_overrides():
         assert kw['bar'] == "baz"
         assert kw['name'] == "name"
 
-def test__stack_broadcast_name_list_overrides():
+def test__double_stack_broadcast_name_list_overrides():
     names = ["aa", "bb", "cc", "dd"]
     stackers = ['a', 'b', 'c', 'd']
-    kws = _stack(stackers, 'start', 'end', foo=[10, 20, 30, 40], bar="baz", name=names)
+    kws = bph._double_stack(stackers, 'start', 'end', foo=[10, 20, 30, 40], bar="baz", name=names)
     assert len(kws) == len(stackers)
     for i, kw in enumerate(kws):
         assert set(['start', 'end', 'foo', 'bar', 'name']) == set(kw.keys())
@@ -175,7 +245,7 @@ def test__graph_will_convert_dataframes_to_sources(pd):
     node_source = pd.DataFrame(data=dict(foo=[]))
     edge_source = pd.DataFrame(data=dict(start=[], end=[], bar=[]))
 
-    kw = _graph(node_source, edge_source)
+    kw = bph._graph(node_source, edge_source)
 
     # 'index' column is added from pandas df
     assert set(kw['node_renderer'].data_source.data.keys()) == {"index", "foo"}
@@ -185,7 +255,7 @@ def test__graph_will_handle_sources_correctly():
     node_source = ColumnDataSource(data=dict(foo=[]))
     edge_source = ColumnDataSource(data=dict(start=[], end=[], bar=[]))
 
-    kw = _graph(node_source, edge_source)
+    kw = bph._graph(node_source, edge_source)
 
     assert set(kw['node_renderer'].data_source.data.keys()) == {"foo"}
     assert set(kw['edge_renderer'].data_source.data.keys()) == {"start", "end", "bar"}
@@ -195,7 +265,7 @@ def test__graph_properly_handle_node_property_mixins():
                   node_nonselection_fill_color="yellow", node_hover_fill_color="red",
                   node_muted_fill_color="orange", node_radius=0.6)
 
-    kw = _graph({}, {}, **kwargs)
+    kw = bph._graph({}, {}, **kwargs)
 
     r = kw['node_renderer']
     assert r.glyph.fill_color == "purple"
@@ -215,7 +285,7 @@ def test__graph_properly_handle_edge_property_mixins():
                   edge_nonselection_line_color="yellow", edge_hover_line_color="red",
                   edge_muted_line_color="orange", edge_line_width=23)
 
-    kw = _graph({}, {}, **kwargs)
+    kw = bph._graph({}, {}, **kwargs)
 
     r = kw['edge_renderer']
     assert r.glyph.line_color == "purple"
@@ -237,121 +307,121 @@ class Test__get_axis_class(object):
     @pytest.mark.parametrize('range', _RANGES)
     @pytest.mark.unit
     def test_axis_type_None(self, range):
-        assert(_get_axis_class(None, range, 0)) == (None, {})
-        assert(_get_axis_class(None, range, 1)) == (None, {})
+        assert(bph._get_axis_class(None, range, 0)) == (None, {})
+        assert(bph._get_axis_class(None, range, 1)) == (None, {})
 
     @pytest.mark.parametrize('range', _RANGES)
     @pytest.mark.unit
     def test_axis_type_linear(self, range):
-        assert(_get_axis_class("linear", range, 0)) == (LinearAxis, {})
-        assert(_get_axis_class("linear", range, 1)) == (LinearAxis, {})
+        assert(bph._get_axis_class("linear", range, 0)) == (LinearAxis, {})
+        assert(bph._get_axis_class("linear", range, 1)) == (LinearAxis, {})
 
     @pytest.mark.parametrize('range', _RANGES)
     @pytest.mark.unit
     def test_axis_type_log(self, range):
-        assert(_get_axis_class("log", range, 0)) == (LogAxis, {})
-        assert(_get_axis_class("log", range, 1)) == (LogAxis, {})
+        assert(bph._get_axis_class("log", range, 0)) == (LogAxis, {})
+        assert(bph._get_axis_class("log", range, 1)) == (LogAxis, {})
 
     @pytest.mark.parametrize('range', _RANGES)
     @pytest.mark.unit
     def test_axis_type_datetime(self, range):
-        assert(_get_axis_class("datetime", range, 0)) == (DatetimeAxis, {})
-        assert(_get_axis_class("datetime", range, 1)) == (DatetimeAxis, {})
+        assert(bph._get_axis_class("datetime", range, 0)) == (DatetimeAxis, {})
+        assert(bph._get_axis_class("datetime", range, 1)) == (DatetimeAxis, {})
 
     @pytest.mark.parametrize('range', _RANGES)
     @pytest.mark.unit
     def test_axis_type_mercator(self, range):
-        assert(_get_axis_class("mercator", range, 0)) == (MercatorAxis, {'dimension': 'lon'})
-        assert(_get_axis_class("mercator", range, 1)) == (MercatorAxis, {'dimension': 'lat'})
+        assert(bph._get_axis_class("mercator", range, 0)) == (MercatorAxis, {'dimension': 'lon'})
+        assert(bph._get_axis_class("mercator", range, 1)) == (MercatorAxis, {'dimension': 'lat'})
 
     def test_axis_type_auto(self):
-        assert(_get_axis_class("auto", FactorRange(), 0)) == (CategoricalAxis, {})
-        assert(_get_axis_class("auto", FactorRange(), 1)) == (CategoricalAxis, {})
-        assert(_get_axis_class("auto", DataRange1d(), 0)) == (LinearAxis, {})
-        assert(_get_axis_class("auto", DataRange1d(), 1)) == (LinearAxis, {})
-        assert(_get_axis_class("auto", Range1d(), 0)) == (LinearAxis, {})
-        assert(_get_axis_class("auto", Range1d(), 1)) == (LinearAxis, {})
-        assert(_get_axis_class("auto", Range1d(start=datetime.datetime(2018, 3, 21)), 0)) == (DatetimeAxis, {})
-        assert(_get_axis_class("auto", Range1d(start=datetime.datetime(2018, 3, 21)), 1)) == (DatetimeAxis, {})
+        assert(bph._get_axis_class("auto", FactorRange(), 0)) == (CategoricalAxis, {})
+        assert(bph._get_axis_class("auto", FactorRange(), 1)) == (CategoricalAxis, {})
+        assert(bph._get_axis_class("auto", DataRange1d(), 0)) == (LinearAxis, {})
+        assert(bph._get_axis_class("auto", DataRange1d(), 1)) == (LinearAxis, {})
+        assert(bph._get_axis_class("auto", Range1d(), 0)) == (LinearAxis, {})
+        assert(bph._get_axis_class("auto", Range1d(), 1)) == (LinearAxis, {})
+        assert(bph._get_axis_class("auto", Range1d(start=datetime.datetime(2018, 3, 21)), 0)) == (DatetimeAxis, {})
+        assert(bph._get_axis_class("auto", Range1d(start=datetime.datetime(2018, 3, 21)), 1)) == (DatetimeAxis, {})
 
 
     @pytest.mark.parametrize('range', _RANGES)
     @pytest.mark.unit
     def test_axis_type_error(self, range):
         with pytest.raises(ValueError):
-            _get_axis_class("junk", range, 0)
+            bph._get_axis_class("junk", range, 0)
         with pytest.raises(ValueError):
-            _get_axis_class("junk", range, 1)
+            bph._get_axis_class("junk", range, 1)
 
 def test__get_scale_numeric_range_linear_axis():
-    s = _get_scale(Range1d(), "linear")
+    s = bph._get_scale(Range1d(), "linear")
     assert isinstance(s, LinearScale)
 
-    s = _get_scale(Range1d(), "datetime")
+    s = bph._get_scale(Range1d(), "datetime")
     assert isinstance(s, LinearScale)
 
-    s = _get_scale(Range1d(), "auto")
+    s = bph._get_scale(Range1d(), "auto")
     assert isinstance(s, LinearScale)
 
 def test__get_scale_numeric_range_log_axis():
-    s = _get_scale(DataRange1d(), "log")
+    s = bph._get_scale(DataRange1d(), "log")
     assert isinstance(s, LogScale)
 
 def test__get_scale_factor_range():
-    s = _get_scale(FactorRange(), "auto")
+    s = bph._get_scale(FactorRange(), "auto")
     assert isinstance(s, CategoricalScale)
 
 def test__get_range_with_None():
-    r = _get_range(None)
+    r = bph._get_range(None)
     assert isinstance(r, DataRange1d)
 
 def test__get_range_with_Range():
     for t in [Range1d, DataRange1d, FactorRange]:
         rng = t()
-        r = _get_range(rng)
+        r = bph._get_range(rng)
         assert r is rng
 
 def test__get_range_with_ndarray():
-    r = _get_range(np.array([10, 20]))
+    r = bph._get_range(np.array([10, 20]))
     assert isinstance(r, Range1d)
     assert r.start == 10
     assert r.end == 20
 
 def test__get_range_with_too_long_ndarray():
     with pytest.raises(ValueError):
-        _get_range(np.array([10, 20, 30]))
+        bph._get_range(np.array([10, 20, 30]))
 
 def test__get_range_with_ndarray_factors():
     f = np.array(["Crosby", "Stills", "Nash", "Young"])
-    r = _get_range(f)
+    r = bph._get_range(f)
     assert isinstance(r, FactorRange)
     assert r.factors == list(f)
 
 def test__get_range_with_series(pd):
-    r = _get_range(pd.Series([20, 30]))
+    r = bph._get_range(pd.Series([20, 30]))
     assert isinstance(r, Range1d)
     assert r.start == 20
     assert r.end == 30
 
 def test__get_range_with_too_long_series(pd):
     with pytest.raises(ValueError):
-        _get_range(pd.Series([20, 30, 40]))
+        bph._get_range(pd.Series([20, 30, 40]))
 
 def test__get_range_with_string_seq():
     f = ["foo" ,"end", "baz"]
     for t in [list, tuple]:
-        r = _get_range(t(f))
+        r = bph._get_range(t(f))
         assert isinstance(r, FactorRange)
         # FactorRange accepts Seq, but _get_range always sets a list copy
         assert r.factors == f
 
 def test__get_range_with_float_bounds():
-    r = _get_range((1.2, 10))
+    r = bph._get_range((1.2, 10))
     assert isinstance(r, Range1d)
     assert r.start == 1.2
     assert r.end == 10
 
-    r = _get_range([1.2, 10])
+    r = bph._get_range([1.2, 10])
     assert isinstance(r, Range1d)
     assert r.start == 1.2
     assert r.end == 10
@@ -359,7 +429,7 @@ def test__get_range_with_float_bounds():
 def test_get_range_with_pandas_group(pd):
     from bokeh.sampledata.iris import flowers
     g = flowers.groupby('species')
-    r = _get_range(g)
+    r = bph._get_range(g)
     assert isinstance(r, FactorRange)
     assert r.factors == ['setosa', 'versicolor', 'virginica'] # should always be sorted
 
@@ -375,12 +445,12 @@ _renderer_args_values = {
     'visible': [None, False, True],
     'muted': [None, False, True]
 }
-@pytest.mark.parametrize('arg,values', [(arg, _renderer_args_values[arg]) for arg in _RENDERER_ARGS])
+@pytest.mark.parametrize('arg,values', [(arg, _renderer_args_values[arg]) for arg in bph._RENDERER_ARGS])
 @pytest.mark.unit
 def test__glyph_receives_renderer_arg(arg, values):
     for value in values:
         with mock.patch('bokeh.plotting.helpers.GlyphRenderer', autospec=True) as gr_mock:
-            fn = _glyph_function(Marker)
+            fn = bph._glyph_function(Marker)
             fn(Figure(), x=0, y=0, **{arg: value})
             _, kwargs = gr_mock.call_args
             assert arg in kwargs and kwargs[arg] == value
