@@ -1,7 +1,13 @@
+#-----------------------------------------------------------------------------
+# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
+#
+# The full license is in the file LICENSE.txt, distributed with this software.
+#-----------------------------------------------------------------------------
 ''' Simplify linking to Bokeh Github resources.
 
-This module proved four new roles that can be uses to easily link
-to various resources in the Bokeh Github repository:
+This module proved four new roles that can be uses to easily link to various
+resources in the Bokeh Github repository:
 
 ``:bokeh-commit:`` : link to a specific commit
 
@@ -14,7 +20,9 @@ to various resources in the Bokeh Github repository:
 Examples
 --------
 
-The following code::
+The following code:
+
+.. code-block:: rest
 
     The repo history shows that :bokeh-commit:`bf19bcb` was made in
     in :bokeh-pull:`1698`, which closed :bokeh-issue:`1694`. This included
@@ -27,12 +35,46 @@ in :bokeh-pull:`1698`,which closed :bokeh-issue:`1694`. This included
 updating all of the files in the :bokeh-tree:`examples` subdirectory.
 
 '''
-from __future__ import absolute_import
 
+#-----------------------------------------------------------------------------
+# Boilerplate
+#-----------------------------------------------------------------------------
+from __future__ import absolute_import, division, print_function, unicode_literals
+
+import logging
+log = logging.getLogger(__name__)
+
+#-----------------------------------------------------------------------------
+# Imports
+#-----------------------------------------------------------------------------
+
+# Standard library imports
 from docutils import nodes, utils
 from docutils.parsers.rst.roles import set_classes
 
-BOKEH_GH = "https://github.com/bokeh/bokeh"
+# External imports
+
+# Bokeh imports
+
+#-----------------------------------------------------------------------------
+# Globals and constants
+#-----------------------------------------------------------------------------
+
+__all__ = (
+    'bokeh_commit',
+    'bokeh_issue',
+    'bokeh_pull',
+    'bokeh_tree',
+    'setup',
+)
+
+#-----------------------------------------------------------------------------
+# General API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Dev API
+#-----------------------------------------------------------------------------
 
 def bokeh_commit(name, rawtext, text, lineno, inliner, options=None, content=None):
     ''' Link to a Bokeh Github issue.
@@ -102,7 +144,6 @@ def bokeh_tree(name, rawtext, text, lineno, inliner, options=None, content=None)
         All of the examples are located in the :bokeh-tree:`examples`
         subdirectory of your Bokeh checkout.
 
-
     Returns 2 part tuple containing list of nodes to insert into the
     document and a list of system messages.  Both are allowed to be
     empty.
@@ -114,13 +155,25 @@ def bokeh_tree(name, rawtext, text, lineno, inliner, options=None, content=None)
     if '-' in tag:
         tag = 'master'
 
-    url = "%s/tree/%s/%s" % (BOKEH_GH, tag, text)
+    url = "%s/tree/%s/%s" % (_BOKEH_GH, tag, text)
     options = options or {}
     set_classes(options)
-    node = nodes.reference(
-        rawtext, text, refuri=url, **options)
+    node = nodes.reference(rawtext, text, refuri=url, **options)
     return [node], []
 
+def setup(app):
+    ''' Required Sphinx extension setup function. '''
+    app.add_role('bokeh-commit', bokeh_commit)
+    app.add_role('bokeh-issue', bokeh_issue)
+    app.add_role('bokeh-pull', bokeh_pull)
+    app.add_role('bokeh-tree', bokeh_tree)
+
+#-----------------------------------------------------------------------------
+# Private API
+#-----------------------------------------------------------------------------
+
+_BOKEH_GH = "https://github.com/bokeh/bokeh"
+
 def _make_gh_link_node(app, rawtext, role, kind, api_type, id, options=None):
     ''' Return a link to a Bokeh Github resource.
 
@@ -134,15 +187,13 @@ def _make_gh_link_node(app, rawtext, role, kind, api_type, id, options=None):
         options (dict) : options dictionary passed to role function
 
     '''
-    url = "%s/%s/%s" % (BOKEH_GH, api_type, id)
+    url = "%s/%s/%s" % (_BOKEH_GH, api_type, id)
     options = options or {}
     set_classes(options)
     node = nodes.reference(
         rawtext, kind + utils.unescape(id), refuri=url, **options)
     return node
 
-def setup(app):
-    app.add_role('bokeh-commit', bokeh_commit)
-    app.add_role('bokeh-issue', bokeh_issue)
-    app.add_role('bokeh-pull', bokeh_pull)
-    app.add_role('bokeh-tree', bokeh_tree)
+#-----------------------------------------------------------------------------
+# Code
+#-----------------------------------------------------------------------------
