@@ -4,7 +4,7 @@
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
 #-----------------------------------------------------------------------------
-""" Generate visual representations of palettes in Bokeh palette groups.
+''' Generate visual representations of palettes in Bokeh palette groups.
 
 The ``bokeh.palettes`` modules expose attributes such as ``mpl``, ``brewer``,
 and ``d3`` that provide groups of palettes. The ``bokeh-palette-group``
@@ -21,7 +21,7 @@ Generates the output:
 
     .. bokeh-palette-group:: mpl
 
-"""
+'''
 
 #-----------------------------------------------------------------------------
 # Boilerplate
@@ -54,9 +54,7 @@ from .templates import PALETTE_GROUP_DETAIL
 __all__ = (
     'bokeh_palette_group',
     'BokehPaletteGroupDirective',
-    'CSS',
     'html_visit_bokeh_palette_group',
-    'JS',
     'setup',
 )
 
@@ -68,15 +66,6 @@ __all__ = (
 # Dev API
 #-----------------------------------------------------------------------------
 
-CSS = """
-<link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.5/css/bootstrap.min.css">
-"""
-
-JS = """
-<script src="https://ajax.googleapis.com/ajax/libs/jquery/1.11.3/jquery.min.js"></script>
-<script src="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.5/js/bootstrap.min.js"></script>
-"""
-
 class bokeh_palette_group(nodes.General, nodes.Element):
     pass
 
@@ -92,7 +81,7 @@ class BokehPaletteGroupDirective(Directive):
         return [node]
 
 def html_visit_bokeh_palette_group(self, node):
-    self.body.append(CSS)
+    self.body.append(_BOOTSTRAP_CSS)
     self.body.append('<div class="container-fluid"><div class="row">"')
     group = getattr(bp, node['group'], None)
     if not isinstance(group, dict):
@@ -105,10 +94,11 @@ def html_visit_bokeh_palette_group(self, node):
         html = PALETTE_GROUP_DETAIL.render(name=name, numbers=numbers, palettes=palettes)
         self.body.append(html)
     self.body.append('</div></div>')
-    self.body.append(JS)
+    self.body.append(_BOOTSTRAP_JS)
     raise nodes.SkipNode
 
 def setup(app):
+    ''' Required Sphinx extension setup function. '''
     app.add_node(bokeh_palette_group, html=(html_visit_bokeh_palette_group, None))
     app.add_directive('bokeh-palette-group', BokehPaletteGroupDirective)
 
@@ -116,6 +106,15 @@ def setup(app):
 # Private API
 #-----------------------------------------------------------------------------
 
+_BOOTSTRAP_CSS = """
+<link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.5/css/bootstrap.min.css">
+"""
+
+_BOOTSTRAP_JS = """
+<script src="https://ajax.googleapis.com/ajax/libs/jquery/1.11.3/jquery.min.js"></script>
+<script src="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.5/js/bootstrap.min.js"></script>
+"""
+
 #-----------------------------------------------------------------------------
 # Code
 #-----------------------------------------------------------------------------
