@@ -27,6 +27,7 @@ import os
 from os.path import dirname, join, abspath, exists, isabs
 import re
 from subprocess import Popen, PIPE
+from collections import OrderedDict
 import sys
 
 # External imports
@@ -80,14 +81,17 @@ class CompilationError(RuntimeError):
     '''
     def __init__(self, error):
         super(CompilationError, self).__init__()
-        self.line = error.get("line")
-        self.column = error.get("column")
-        self.message = error.get("message")
-        self.text = error.get("text")
-        self.annotated = error.get("annotated")
+        if isinstance(error, dict):
+            self.line = error.get("line")
+            self.column = error.get("column")
+            self.message = error.get("message")
+            self.text = error.get("text")
+            self.annotated = error.get("annotated")
+        else:
+            self.text = error
 
     def __str__(self):
-        return self.text
+        return "\n" + self.text.strip()
 
 bokehjs_dir = settings.bokehjsdir()
 nodejs_min_version = (6, 10, 0)
@@ -99,9 +103,23 @@ def npmjs_version():
     return _version(_run_npmjs)
 
 def nodejs_compile(code, lang="javascript", file=None):
+    if lang == "coffeescript":
+        from bokeh.util.deprecation import deprecated
+        deprecated("CoffeeScript support is deprecated and will be removed in an eventual 2.0 release. "
+                   "Use JavaScript or TypeScript directly instead.")
     compilejs_script = join(bokehjs_dir, "js", "compiler.js")
-    output = _run_nodejs([compilejs_script], dict(code=code, lang=lang, file=file))
-    return AttrDict(json.loads(output))
+    output = _run_nodejs([compilejs_script], dict(code=code, lang=lang, file=file, bokehjs_dir=bokehjs_dir))
+    lines = output.split("\n")
+    for i, line in enumerate(lines):
+        if not line.startswith("LOG"):
+            break
+        else:
+            print(line)
+    obj = json.loads("\n".join(lines[i:]))
+    if isinstance(obj, dict):
+        return AttrDict(obj)
+    else:
+        return dict(error=obj)
 
 class Implementation(object):
     ''' Base class for representing Bokeh custom model implementations.
@@ -145,6 +163,12 @@ class CoffeeScript(Inline):
 
     '''
 
+    def __init__(self, *args, **kw):
+        from bokeh.util.deprecation import deprecated
+        deprecated("CoffeeScript support is deprecated and will be removed in an eventual 2.0 release. "
+                   "Use JavaScript or TypeScript directly instead.")
+        super(CoffeeScript, self).__init__(*args, **kw)
+
     @property
     def lang(self):
         return "coffeescript"
@@ -172,7 +196,7 @@ class JavaScript(Inline):
         .. code-block:: python
 
             class MyExt(Model):
-                __implementation__ = Javacript(""" <JavaScript code> """)
+                __implementation__ = JavaScript(""" <JavaScript code> """)
 
     '''
     @property
@@ -261,7 +285,8 @@ class CustomModel(object):
                 impl = CoffeeScript(impl)
 
         if isinstance(impl, Inline) and impl.file is None:
-            impl = impl.__class__(impl.code, (self.file or "<string>") + ":" + self.name)
+            file = "%s%s.ts" % (self.file + ":" if self.file else "", self.name)
+            impl = impl.__class__(impl.code, file)
 
         return impl
 
@@ -284,102 +309,7 @@ def set_cache_hook(hook):
     global _CACHING_IMPLEMENTATION
     _CACHING_IMPLEMENTATION = hook
 
-def bundle_models(models):
-    """Create a bundle of `models`. """
-
-    custom_models = _get_custom_models(models)
-    if custom_models is None:
-        return
-
-    exports = []
-    modules = []
-
-    def read_json(name):
-        with io.open(join(bokehjs_dir, "js", name + ".json"), encoding="utf-8") as f:
-            return json.loads(f.read())
-
-    bundles = ["bokeh", "bokeh-api", "bokeh-widgets", "bokeh-tables", "bokeh-gl"]
-    known_modules = set(sum([ read_json(name) for name in bundles ], []))
-    custom_impls = _compile_models(custom_models)
-
-    extra_modules = {}
-
-    def resolve_modules(to_resolve, root):
-        resolved = {}
-        for module in to_resolve:
-            if module.startswith(("./", "../")):
-                def mkpath(module, ext=""):
-                    return abspath(join(root, *module.split("/")) + ext)
-
-                if module.endswith(exts):
-                    path = mkpath(module)
-                    if not exists(path):
-                        raise RuntimeError("no such module: %s" % module)
-                else:
-                    for ext in exts:
-                        path = mkpath(module, ext)
-                        if exists(path):
-                            break
-                    else:
-                        raise RuntimeError("no such module: %s" % module)
-
-                impl = FromFile(path)
-                compiled = nodejs_compile(impl.code, lang=impl.lang, file=impl.file)
-
-                if "error" in compiled:
-                    raise CompilationError(compiled.error)
-
-                if impl.lang == "less":
-                    code = _style_template % dict(css=json.dumps(compiled.code))
-                    deps = []
-                else:
-                    code = compiled.code
-                    deps = compiled.deps
-
-                sig = hashlib.sha256(code.encode('utf-8')).hexdigest()
-                resolved[module] = sig
-
-                deps_map = resolve_deps(deps, dirname(path))
-
-                if sig not in extra_modules:
-                    extra_modules[sig] = True
-                    modules.append((sig, code, deps_map))
-            else:
-                raise RuntimeError("no such module: %s" % module)
-
-        return resolved
-
-    def resolve_deps(deps, root):
-        custom_modules = set(model.module for model in custom_models.values())
-        missing = set(deps) - known_modules - custom_modules
-        return resolve_modules(missing, root)
-
-    for model in custom_models.values():
-        compiled = custom_impls[model.full_name]
-        deps_map = resolve_deps(compiled.deps, model.path)
-
-        exports.append((model.name, model.module))
-        modules.append((model.module, compiled.code, deps_map))
-
-    # sort everything by module name
-    exports = sorted(exports, key=lambda spec: spec[1])
-    modules = sorted(modules, key=lambda spec: spec[0])
-
-    for i, (module, code, deps) in enumerate(modules):
-        for name, ref in deps.items():
-            code = code.replace("""require("%s")""" % name, """require("%s")""" % ref)
-            code = code.replace("""require('%s')""" % name, """require('%s')""" % ref)
-        modules[i] = (module, code)
-
-    sep = ",\n"
-
-    exports = sep.join(_export_template % dict(name=name, module=module) for (name, module) in exports)
-    modules = sep.join(_module_template % dict(module=module, source=code) for (module, code) in modules)
-
-    content = _plugin_template % dict(prelude=_plugin_prelude, exports=exports, modules=modules)
-    return _plugin_umd % dict(content=content)
-
-def calc_cache_key():
+def calc_cache_key(custom_models):
     ''' Generate a key to cache a custom extension implementation with.
 
     There is no metadata other than the Model classes, so this is the only
@@ -389,29 +319,33 @@ def calc_cache_key():
     not ideal but possibly a better solution can be found found later.
 
     '''
-
-    models = Model.model_class_reverse_map.values()
-    custom_model_names = ""
-
-    for cls in models:
-        impl = getattr(cls, "__implementation__", None)
-
-        if impl is not None:
-            model = CustomModel(cls)
-            custom_model_names += model.full_name
-
-    key = hashlib.sha256(custom_model_names.encode('utf-8')).hexdigest()
-    return key
+    model_names = {model.full_name for model in custom_models.values()}
+    encoded_names = ",".join(sorted(model_names)).encode('utf-8')
+    return hashlib.sha256(encoded_names).hexdigest()
 
 _bundle_cache = {}
 
-def bundle_all_models():
-    key = calc_cache_key()
+def bundle_models(models):
+    """Create a bundle of selected `models`. """
+    custom_models = _get_custom_models(models)
+    if custom_models is None:
+        return None
+
+    key = calc_cache_key(custom_models)
     bundle = _bundle_cache.get(key, None)
     if bundle is None:
-        _bundle_cache[key] = bundle = bundle_models(Model.model_class_reverse_map.values()) or ""
+        try:
+            _bundle_cache[key] = bundle = _bundle_models(custom_models)
+        except CompilationError as error:
+            print("Compilation failed:", file=sys.stderr)
+            print(str(error), file=sys.stderr)
+            sys.exit(1)
     return bundle
 
+def bundle_all_models():
+    """Create a bundle of all models. """
+    return bundle_models(None)
+
 #-----------------------------------------------------------------------------
 # Dev API
 #-----------------------------------------------------------------------------
@@ -540,7 +474,7 @@ def _run(app, argv, input=None):
     (stdout, errout) = proc.communicate(input=None if input is None else json.dumps(input).encode())
 
     if proc.returncode != 0:
-        raise RuntimeError(errout)
+        raise RuntimeError(errout.decode('utf-8'))
     else:
         return _crlf_cr_2_lf(stdout.decode('utf-8'))
 
@@ -566,7 +500,10 @@ _CACHING_IMPLEMENTATION = _model_cache_no_op
 
 def _get_custom_models(models):
     """Returns CustomModels for models with a custom `__implementation__`"""
-    custom_models = {}
+    if models is None:
+        models = Model.model_class_reverse_map.values()
+
+    custom_models = OrderedDict()
     for cls in models:
         impl = getattr(cls, "__implementation__", None)
 
@@ -604,6 +541,98 @@ def _compile_models(custom_models):
 
     return custom_impls
 
+def _bundle_models(custom_models):
+    """ Create a JavaScript bundle with selected `models`. """
+    exports = []
+    modules = []
+
+    def read_json(name):
+        with io.open(join(bokehjs_dir, "js", name + ".json"), encoding="utf-8") as f:
+            return json.loads(f.read())
+
+    bundles = ["bokeh", "bokeh-api", "bokeh-widgets", "bokeh-tables", "bokeh-gl"]
+    known_modules = set(sum([ read_json(name) for name in bundles ], []))
+    custom_impls = _compile_models(custom_models)
+
+    extra_modules = {}
+
+    def resolve_modules(to_resolve, root):
+        resolved = {}
+        for module in to_resolve:
+            if module.startswith(("./", "../")):
+                def mkpath(module, ext=""):
+                    return abspath(join(root, *module.split("/")) + ext)
+
+                if module.endswith(exts):
+                    path = mkpath(module)
+                    if not exists(path):
+                        raise RuntimeError("no such module: %s" % module)
+                else:
+                    for ext in exts:
+                        path = mkpath(module, ext)
+                        if exists(path):
+                            break
+                    else:
+                        raise RuntimeError("no such module: %s" % module)
+
+                impl = FromFile(path)
+                compiled = nodejs_compile(impl.code, lang=impl.lang, file=impl.file)
+
+                if "error" in compiled:
+                    raise CompilationError(compiled.error)
+
+                if impl.lang == "less":
+                    code = _style_template % dict(css=json.dumps(compiled.code))
+                    deps = []
+                else:
+                    code = compiled.code
+                    deps = compiled.deps
+
+                sig = hashlib.sha256(code.encode('utf-8')).hexdigest()
+                resolved[module] = sig
+
+                deps_map = resolve_deps(deps, dirname(path))
+
+                if sig not in extra_modules:
+                    extra_modules[sig] = True
+                    modules.append((sig, code, deps_map))
+            else:
+                index = module + ("" if module.endswith("/") else "/") + "index"
+                if index not in known_modules:
+                    raise RuntimeError("no such module: %s" % module)
+
+        return resolved
+
+    def resolve_deps(deps, root):
+        custom_modules = set(model.module for model in custom_models.values())
+        missing = set(deps) - known_modules - custom_modules
+        return resolve_modules(missing, root)
+
+    for model in custom_models.values():
+        compiled = custom_impls[model.full_name]
+        deps_map = resolve_deps(compiled.deps, model.path)
+
+        exports.append((model.name, model.module))
+        modules.append((model.module, compiled.code, deps_map))
+
+    # sort everything by module name
+    exports = sorted(exports, key=lambda spec: spec[1])
+    modules = sorted(modules, key=lambda spec: spec[0])
+
+    for i, (module, code, deps) in enumerate(modules):
+        for name, ref in deps.items():
+            code = code.replace("""require("%s")""" % name, """require("%s")""" % ref)
+            code = code.replace("""require('%s')""" % name, """require('%s')""" % ref)
+        modules[i] = (module, code)
+
+    sep = ",\n"
+
+    exports = sep.join(_export_template % dict(name=name, module=module) for (name, module) in exports)
+    modules = sep.join(_module_template % dict(module=module, source=code) for (module, code) in modules)
+
+    content = _plugin_template % dict(prelude=_plugin_prelude, exports=exports, modules=modules)
+    return _plugin_umd % dict(content=content)
+
 #-----------------------------------------------------------------------------
 # Code
 #-----------------------------------------------------------------------------
